//----------------------------------------------------------------------------------
// File:   InstancingTests.cpp
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "DXUT.h"
#pragma warning (disable:4996 4995 4127)
#include "DXUTgui.h"
#include <string>
#include "resource.h"
#include "InstancingTests.h"
#include <vector>
#include <list>
#include "stdio.h"
#include <iostream>
#include <fstream>

#include "SDKmisc.h"

#include "InstancingTestHarness.h"

InstancingTestHarness::InstancingTestHarness()
{
    m_currentEvent = 0;
    m_accumFrame = 0;
    m_accumTime = 0.f;
    m_FrameSmoothingFactor = 200;
}

void InstancingTestHarness::InitializeExperiments(InstancingTests*pIT, 
                                                  DWORD maxInstances, DWORD maxLoad, 
                                                  DWORD instSteps, DWORD loadSteps, 
                                                  bool bLogIncrement, int frameSmooth,
                                                  LPCWSTR wzTag)
{
    if(!pIT) return;
    assert(pIT);

    m_complete = false;
    m_FrameSmoothingFactor = frameSmooth;

    // Create a new output filename
    char szTag[MAX_PATH];
    WideCharToMultiByte(CP_ACP,0,wzTag,-1,szTag,MAX_PATH," ",FALSE);
    sprintf(m_szFilename,"InstancingTests_%s_i%d_l%d.csv",szTag,maxInstances,maxLoad);

    // Create our script of simulation
    for(unsigned char useinstancing = 0;useinstancing <= 2;useinstancing++)
    {
        DWORD numInstances = 1;
        DWORD instIncr = max(1,maxInstances / instSteps); // $ only used for linear increments
        DWORD loadIncr = max(1,maxLoad / loadSteps);

        if(bLogIncrement)
        {
            instIncr = 1;   // updated on the fly
        }
        while(1)
        {
            // Load simulation is always linear progression
            for(DWORD cpuload=0;cpuload<=maxLoad;cpuload+=loadIncr)                       
            {
                SimulationTestEvent e;
                e.cpuload = cpuload;
                e.instances = numInstances;
                e.useinstancing = useinstancing;
                m_events.push_back(e);
            }

            // linear withing a scale, and then log up.  Gets some local trends as well as covers a larger area in a small number of samples.
            if(bLogIncrement && numInstances >= 10*instIncr)
            {
                instIncr *= 10;
            }

            if(numInstances >= maxInstances) // we did the final run, so break out.
                break;

            numInstances += instIncr; // advance

            if(numInstances > maxInstances) // clamp to ensure we get a data point on the final target
                numInstances = maxInstances;
        }
    }
}

void InstancingTestHarness::FrameRender(InstancingTests*pIT,float fElapsedTime)
{
    if(m_complete) return;

    // each simulation event smooths over a few frames...
    m_accumTime += fElapsedTime;
    m_accumFrame++;

    // each simulation event smooths over a few frames...
    if(m_accumFrame < m_FrameSmoothingFactor) return;

    SimulationTestResult r;
    r.cpuload = m_events[m_currentEvent].cpuload;
    r.instances = m_events[m_currentEvent].instances;
    r.useinstancing = m_events[m_currentEvent].useinstancing;
    r.ms = m_accumTime/(float)(m_FrameSmoothingFactor);
    m_results.push_back(r);

    m_accumFrame = 0;
    m_accumTime = 0.f;

    // Go to the next event
    m_currentEvent++;

    if(m_currentEvent == (int)m_events.size()) 
    {
        WriteOutData(pIT);
        m_complete = true;
        return;
    }


    // Update the system with new simulation event config
    pIT->SetCPULoad(m_events[m_currentEvent].cpuload);
    pIT->SetNumInstanced(m_events[m_currentEvent].instances);
    pIT->SetUseInstancing(m_events[m_currentEvent].useinstancing>0);
    pIT->SetTraditionalInstancing(m_events[m_currentEvent].useinstancing>1);

}

void InstancingTestHarness::FrameUpdate(InstancingTests*pIT, float fElapsedTime)
{
}

/*
    Dump each batch run into the output file.
*/
void InstancingTestHarness::WriteOutData(InstancingTests*pIT)
{
    std::string output;

    // You can think of the cpuload as X axis, num instances as Y axis and type of instancing as z axis.
    char szLine[MAX_PATH];

    unsigned int currentResult = 0;

    // 3 types of rendering 0=single draw, 1=constants instanciton, 2=traditional instancing...
    for(int useinstancing = 0;useinstancing <= 2;useinstancing++)
    {
        switch(useinstancing)
        {
        case 0:
            sprintf(szLine,"Instancing = None\n");
            break;
        case 1:
            sprintf(szLine,"Instancing = Constants Instancing (SV_InstanceID)\n");
            break;
        default:
            sprintf(szLine,"Instancing = Traditional Stream Instancing\n");
            break;
        }
        
        output.append(szLine);

        // effectively here we write out an X,Y slice of the result data

        // Header, write out columns as cpu load, but onyl for the first instancetype,instances group.
        unsigned int headerCurrent = currentResult;
        output.append("#Inst/Load");
        while(headerCurrent < m_results.size() && 
            m_results[headerCurrent].instances == m_results[currentResult].instances && 
            m_results[headerCurrent].useinstancing == m_results[currentResult].useinstancing)
        {
            sprintf(szLine,",%d",m_results[headerCurrent].cpuload);
            output.append(szLine);

            headerCurrent++;
        }
        output.append("\n");

        while(currentResult < m_results.size() && m_results[currentResult].useinstancing == useinstancing)
        {
            DWORD numInstances = m_results[currentResult].instances;
            DWORD useInstancing = m_results[currentResult].useinstancing;

            // Prep each line with current CPU load
            sprintf(szLine,"%d",numInstances);
            output.append(szLine);


            // then add in columns for all instance counts...
            while(currentResult < m_results.size() && m_results[currentResult].instances == numInstances && m_results[currentResult].useinstancing == useInstancing)
            {
                sprintf(szLine,",%f",m_results[currentResult].ms);
                output.append(szLine);

                currentResult++;    // we've outputted the value, next result
            }
            output.append("\n");
        }
    }

    FILE * pFile;
    pFile = fopen (m_szFilename,"wt");
    if (pFile!=NULL)
    {
        fputs (output.c_str(),pFile);
        fclose (pFile);
    }
}
