//----------------------------------------------------------------------------------
// File:   InstancingTests.cpp
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "DXUT.h"
#include "SDKmisc.h"
#include "DXUTcamera.h"
#include "sdkmesh_old.h"
#include "InstancingTests.h"
#include "TextureLibray.h"
#include <string>

inline float randRange(float min, float max)
{
    return min + (max-min)*(rand()/32768.f);
}

InstancingTests::InstancingTests()
{
	memset(m_instanceData,0,INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES*sizeof(InstanceData));
	m_pd3dDevice = NULL;
	m_pInputLayout = NULL;
    m_pInstancedInputLayout = NULL;
	m_pMeshBuffer = NULL;
	m_pMeshIndexBuffer = NULL;
	m_pTechInstanced = NULL;
	m_pTechNormal = NULL;
    m_bUseInstancing = true;
    m_bTraditionalInstancing = false;
    m_bMoveInstances = true;
    m_numInstances = 1000;
    m_bInstanceBufferNeedsUpdate = true;
    m_pDiffuseTexture = NULL;
    m_pDiffuseSRV = NULL;
    m_pNormalTexture = NULL;
    m_pNormalSRV = NULL;
    m_fScale = 1.f;
    m_cpuLoad = 0;
}

InstancingTests::~InstancingTests()
{
}

void InstancingTests::LoadNewMesh(LPCWSTR filename)
{
    HRESULT hr;

    if(!m_pd3dDevice) return;   // can't make any meshes until we have been initialized

    SAFE_RELEASE(m_pMeshBuffer);
    SAFE_RELEASE(m_pMeshIndexBuffer);

	//-----------------------------------------------------------------------------------------------
	// Use DXUT utility mesh to load some terrain geometry
	//CDXUTMesh10	Mesh;
    WCHAR str[MAX_PATH];
    V( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, filename ) );
	Mesh.Create(m_pd3dDevice,str,(D3D10_INPUT_ELEMENT_DESC*)VertexLayout,sizeof(VertexLayout)/sizeof(D3D10_INPUT_ELEMENT_DESC));

	m_numVertices = Mesh.m_pMesh10->GetVertexCount();
	m_numIndices = Mesh.m_pMesh10->GetFaceCount()*3;

	//-----------------------------------------------------------------------------------------------	
	// Pull out buffers for use during instancing.
	void *pData = NULL;
	SIZE_T loadingSize = 0;

	// VB
	assert(Mesh.m_pMesh10->GetVertexBufferCount() == 1);
	ID3DX10MeshBuffer *pSourceVertexBuffer = NULL;
	V(Mesh.m_pMesh10->GetVertexBuffer(0,&pSourceVertexBuffer));
	pSourceVertexBuffer->Map(&pData,&loadingSize);
	if(loadingSize != m_numVertices*sizeof(Vertex)) V(E_FAIL);

	D3D10_BUFFER_DESC bufferDesc;
	ZeroMemory(&bufferDesc,sizeof(D3D10_BUFFER_DESC));
	bufferDesc.ByteWidth = m_numVertices*sizeof(Vertex);
	bufferDesc.Usage = D3D10_USAGE_DEFAULT;
	bufferDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;

	D3D10_SUBRESOURCE_DATA subresourceData;
	subresourceData.pSysMem = (void*)pData;
	V( m_pd3dDevice->CreateBuffer( &bufferDesc, &subresourceData, &m_pMeshBuffer ) );

    pSourceVertexBuffer->Unmap();
    SAFE_RELEASE(pSourceVertexBuffer);

	// the IB
	ID3DX10MeshBuffer *pSourceIndexBuffer = NULL;
	Mesh.m_pMesh10->GetIndexBuffer(&pSourceIndexBuffer);
	pSourceIndexBuffer->Map(&pData,&loadingSize);
	if(loadingSize != m_numIndices*sizeof(DWORD)) V(E_FAIL);

	ZeroMemory(&bufferDesc,sizeof(D3D10_BUFFER_DESC));
	bufferDesc.ByteWidth = m_numIndices*sizeof(DWORD);
	bufferDesc.Usage = D3D10_USAGE_DEFAULT;
	bufferDesc.BindFlags = D3D10_BIND_INDEX_BUFFER;

	subresourceData.pSysMem = (void*)pData;
	V( m_pd3dDevice->CreateBuffer( &bufferDesc, &subresourceData, &m_pMeshIndexBuffer ) );

	pSourceIndexBuffer->Unmap();
	SAFE_RELEASE(pSourceIndexBuffer);

    SAFE_RELEASE(m_pDiffuseTexture);
    SAFE_RELEASE(m_pDiffuseSRV);
    SAFE_RELEASE(m_pNormalTexture);
    SAFE_RELEASE(m_pNormalSRV);

    LoadTexture(L"../Media/cloth.dds",&m_pDiffuseTexture,&m_pDiffuseSRV);
    LoadTexture(L"../Media/cloth_normal.dds",&m_pNormalTexture,&m_pNormalSRV);

    assert(m_pDiffuse);
    m_pDiffuse->SetResource(m_pDiffuseSRV);
    m_pNormalMap->SetResource(m_pNormalSRV);

    Mesh.Destroy();
}

HRESULT InstancingTests::Initialize(ID3D10Device* pd3dDevice)
{
	HRESULT hr;
	m_pd3dDevice = pd3dDevice; m_pd3dDevice->AddRef();	// Grab us our own copy

	//-----------------------------------------------------------------------------------------------
	// Load our FX and technique, and create input layout variable
    LPCSTR sMaxInstanceConsts = new CHAR[MAX_PATH];
    sprintf_s((char*)sMaxInstanceConsts,MAX_PATH,"%d",INTERNAL_MAX_INSTANCES);

	D3D10_SHADER_MACRO mac[2] =
	{
        { "MAX_INSTANCE_CONSTANTS"  , sMaxInstanceConsts},
		{ NULL                      , NULL }
	};

	WCHAR str[MAX_PATH];
	V( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"InstancingTests.fx" ) );

	V( D3DX10CreateEffectFromFile( str, mac, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, &m_pEffect10, NULL, &hr ) );

	m_pTechInstanced = m_pEffect10->GetTechniqueByName("RenderSceneInstanced");
    m_pTechInstancedConstants = m_pEffect10->GetTechniqueByName("RenderSceneInstancedConstants");
	m_pTechNormal = m_pEffect10->GetTechniqueByName("RenderSceneNormal");

	m_pWorldViewProjectionVar = m_pEffect10->GetVariableBySemantic( "WorldViewProjection" )->AsMatrix();
	m_pWorldVar = m_pEffect10->GetVariableBySemantic( "World" )->AsMatrix();
    m_pDiffuse = m_pEffect10->GetVariableBySemantic( "DIFFUSE" )->AsShaderResource();
    m_pNormalMap = m_pEffect10->GetVariableBySemantic( "NORMALMAP" )->AsShaderResource();
	m_pInstanceDataConstBuffer = m_pEffect10->GetConstantBufferByName("cInstanceData");

	D3D10_PASS_DESC PassDesc;
	V( m_pTechInstancedConstants->GetPassByIndex( 0 )->GetDesc( &PassDesc ) );
	V( pd3dDevice->CreateInputLayout( VertexLayout, sizeof(VertexLayout)/sizeof(D3D10_INPUT_ELEMENT_DESC), 
		PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pInputLayout ) );

	V( m_pTechInstanced->GetPassByIndex( 0 )->GetDesc( &PassDesc ) );
	V( pd3dDevice->CreateInputLayout( InstancedVertexLayout, sizeof(InstancedVertexLayout)/sizeof(D3D10_INPUT_ELEMENT_DESC), 
		PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pInstancedInputLayout ) );


    D3DXMATRIX scale;
    D3DXMATRIX rotation;
    D3DXMATRIX translation;
    for(int i=0;i<INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES;i++)
    {
        D3DXMatrixScaling(&scale,1,1,1);
        if(i==0)
            D3DXMatrixIdentity(&translation);
        else
            D3DXMatrixTranslation(&translation,randRange(-160,160),randRange(-160,160),randRange(-160,160));
        D3DXMatrixIdentity(&rotation);

        m_initialMatrix[i] = scale * rotation * translation ;
        SetWorldMatrixFor(i,m_initialMatrix[i]);
    }

    //-----------------------------------------------------------------------------------------------
    // Create a buffer to hold our per instance data and be bound to a constant buffer perhaps or a texture buffer...
    CreateInstanceBuffers();

	return S_OK;
}

HRESULT InstancingTests::Update(float time)
{
    if(!m_pd3dDevice) return E_FAIL;


    // Simulate other game related load $$ This is purely for test purposes
    if(m_cpuLoad > 0)
    {
        DWORD start = GetTickCount();
        while(GetTickCount() - start < (DWORD)m_cpuLoad)
        {
            // silly op that will never add anything to the DWORD
            start += (unsigned int)(0.000001f*(0.0000001f * ((float)NumInstances())));
        }
    }

    if(m_bMoveInstances)
    {
        D3DXMATRIX world;
        D3DXMATRIX scale;
        D3DXMATRIX rotation;
        D3DXMATRIX translation;
        for(int i=0;i<m_numInstances;i++)
        {
            D3DXMatrixTranslation(&translation,0,0.5f*sin(time*(3.141592654f/2.f + ((float)i / m_numInstances)*3.141592f)),0);
            D3DXMATRIX mScale;
            D3DXMatrixScaling(&mScale,m_fScale,m_fScale,m_fScale);
            D3DXMATRIX mNewWorld = mScale * translation * m_initialMatrix[i];

            SetWorldMatrixFor(i,mNewWorld);
        }
        m_bInstanceBufferNeedsUpdate = true;
    }
	return S_OK;
}

void InstancingTests::SetScale(float scale)
{
    if(!m_bMoveInstances)   // one time update if the per frame update isn't running....
    {
        for(int i=0;i<m_numInstances;i++)
        {
            D3DXMATRIX mOldWorld = WorldMatrixFor(i);

            D3DXMATRIX mScale;
            D3DXMatrixScaling(&mScale,scale/m_fScale,scale/m_fScale,scale/m_fScale);
            D3DXMATRIX mNewWorld = mScale * mOldWorld;
            SetWorldMatrixFor(i,mNewWorld);
        }

        m_bInstanceBufferNeedsUpdate = true;
    }

    m_fScale = scale;
}

unsigned int InstancingTests::Render(ID3D10Device* pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time)
{
	HRESULT hr;
    if(!m_pd3dDevice) return 0;

    D3DXVECTOR3 viewPos = D3DXVECTOR3(mView._41,mView._42,mView._43);
    m_pEffect10->GetVariableBySemantic("VIEWPOSITION")->AsVector()->SetFloatVector((float*)&viewPos);

	pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

    unsigned int drawCalls = 0;
	if(m_bUseInstancing)
	{
	    // Update instance buffer with possibly new simulation info
        if(m_bInstanceBufferNeedsUpdate)
        {
            V(UpdateInstanceData(time));
            m_bInstanceBufferNeedsUpdate = false;
        }

        if(m_bTraditionalInstancing)
		    drawCalls  = DrawStreamInstanced(pd3dDevice,mView,mProj,time);
        else
            drawCalls  = DrawConstantsInstanced(pd3dDevice,mView,mProj,time);
	}
	else
	{
		drawCalls  = Draw(pd3dDevice,mView,mProj,time);
	}

	return drawCalls;
}

unsigned int  InstancingTests::Draw(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time)
{
	pd3dDevice->IASetInputLayout(m_pInputLayout);

	UINT uiVertexStride = sizeof(Vertex);
	UINT uOffset = 0;
	pd3dDevice->IASetVertexBuffers(0,1,&m_pMeshBuffer, &uiVertexStride, &uOffset );
	pd3dDevice->IASetIndexBuffer(m_pMeshIndexBuffer,DXGI_FORMAT_R32_UINT,0);

    // Ensure no second buffer left over from instancing...
    uiVertexStride = 0;
    ID3D10Buffer *pNullBuff = NULL;
    pd3dDevice->IASetVertexBuffers(1,1,&pNullBuff, &uiVertexStride, &uOffset );

    // iterate over the # of charactes to draw
	int realNumToDraw = min(INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES,m_numInstances);
    unsigned int drawCalls = 0;
	for(int currentInstance = 0;currentInstance<realNumToDraw;currentInstance++)
	{
		D3DXMATRIX mWorld = WorldMatrixFor(currentInstance);
		D3DXMATRIX mWorldViewProj = mWorld * mView * mProj;
		m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);
		m_pWorldVar->SetMatrix((float*)mWorld);

		// Apply the technique contained in the effect
		D3D10_TECHNIQUE_DESC techDesc;
		memset( &techDesc, 0, sizeof( D3D10_TECHNIQUE_DESC ) );
		m_pTechNormal->GetDesc( &techDesc );
		for (UINT iPass = 0; iPass < techDesc.Passes; iPass++)
		{
			ID3D10EffectPass *pCurrentPass = m_pTechNormal->GetPassByIndex( iPass );
			pCurrentPass->Apply( 0 );
			pd3dDevice->DrawIndexed(m_numIndices,0,0);
            drawCalls++;
		}
	}
	return drawCalls;
}

/*
    Draw the test scene using stream instancing.  This means pushing the per instance data into a second vertex stream.
*/
unsigned int  InstancingTests::DrawStreamInstanced(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time)
{
	pd3dDevice->IASetInputLayout(m_pInstancedInputLayout);

    ID3D10Buffer *vertBuffers[2] = {m_pMeshBuffer,m_pTotalInstanceDataBuffer};
	ID3D10Buffer *indexBuffer = m_pMeshIndexBuffer;

    UINT uiVertexStrides[2] = {sizeof(Vertex),sizeof(InstanceData)};
    UINT uOffsets[2] = {0,0};

	pd3dDevice->IASetVertexBuffers(0,2,vertBuffers, uiVertexStrides, uOffsets );
	pd3dDevice->IASetIndexBuffer(indexBuffer,DXGI_FORMAT_R32_UINT,0);

    D3DXMATRIX mWorldViewProj = mView * mProj;
    m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);

    unsigned int drawCalls = 0;

    // Apply the technique contained in the effect, and then draw!
    D3D10_TECHNIQUE_DESC techDesc;
    memset( &techDesc, 0, sizeof( D3D10_TECHNIQUE_DESC ) );
    m_pTechInstanced->GetDesc( &techDesc );
    for (UINT iPass = 0; iPass < techDesc.Passes; iPass++)
    {
	    ID3D10EffectPass *pCurrentPass = m_pTechInstanced->GetPassByIndex( iPass );
	    pCurrentPass->Apply( 0 );
	    pd3dDevice->DrawIndexedInstanced(m_numIndices,m_numInstances,0,0,0);
        drawCalls++;
    }

	return drawCalls;
}

/*
    Draw the test scene using constant instancing.  This means pushing the per instance data into shader constant memory.
*/
unsigned int  InstancingTests::DrawConstantsInstanced(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time)
{
	pd3dDevice->IASetInputLayout(m_pInputLayout);

	ID3D10Buffer *vertBuffer = m_pMeshBuffer;
	ID3D10Buffer *indexBuffer = m_pMeshIndexBuffer;

	UINT uiVertexStride = sizeof(Vertex);
	UINT uOffset = 0;

	pd3dDevice->IASetVertexBuffers(0,1,&vertBuffer, &uiVertexStride, &uOffset );
	pd3dDevice->IASetIndexBuffer(indexBuffer,DXGI_FORMAT_R32_UINT,0);

    uiVertexStride = 0;
    ID3D10Buffer *pNullBuff = NULL;
    pd3dDevice->IASetVertexBuffers(1,1,&pNullBuff, &uiVertexStride, &uOffset );

    ID3D10Buffer *pEffectConstantBuffer = NULL;
    m_pInstanceDataConstBuffer->GetConstantBuffer(&pEffectConstantBuffer);

    D3DXMATRIX mWorldViewProj = mView * mProj;
    m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);

    unsigned int drawCalls = 0;
    unsigned int buffersUsed = (m_numInstances / INTERNAL_MAX_INSTANCES)+1;
    for(unsigned int buffer = 0;buffer < buffersUsed ; buffer++)
    {
        int numInstances = INTERNAL_MAX_INSTANCES;
        if(buffer == buffersUsed - 1)       // restrict drawing all instances in the final buffer to get an exact count
            numInstances = m_numInstances % INTERNAL_MAX_INSTANCES;

        D3D10_BUFFER_DESC srcDesc;
        D3D10_BUFFER_DESC dstDesc;
        m_pInstanceDataBuffer[buffer]->GetDesc(&srcDesc);
        pEffectConstantBuffer->GetDesc(&dstDesc);
        pd3dDevice->CopyResource(pEffectConstantBuffer,m_pInstanceDataBuffer[buffer]);

	    // Apply the technique contained in the effect, and then draw!
	    D3D10_TECHNIQUE_DESC techDesc;
	    memset( &techDesc, 0, sizeof( D3D10_TECHNIQUE_DESC ) );
	    m_pTechInstancedConstants->GetDesc( &techDesc );
	    for (UINT iPass = 0; iPass < techDesc.Passes; iPass++)
	    {
		    ID3D10EffectPass *pCurrentPass = m_pTechInstancedConstants->GetPassByIndex( iPass );
		    pCurrentPass->Apply( 0 );
		    pd3dDevice->DrawIndexedInstanced(m_numIndices,numInstances,0,0,0);
            drawCalls++;
	    }
    }

    SAFE_RELEASE(pEffectConstantBuffer);

	return drawCalls;
}


HRESULT InstancingTests::CreateInstanceBuffers()
{
	HRESULT hr = S_OK;

	// Create a	resource with the input	matrices
	D3D10_BUFFER_DESC bufferDesc =
	{
		INTERNAL_MAX_INSTANCES * sizeof( InstanceData ),
		D3D10_USAGE_DEFAULT,
		D3D10_BIND_VERTEX_BUFFER|D3D10_BIND_SHADER_RESOURCE,
		0,
		0
	};

    // Make a series of small buffers to be used for constants based instancing.
    for(int i=0;i<INTERNAL_MAX_BUFFERS;i++)
	    V(m_pd3dDevice->CreateBuffer( &bufferDesc, NULL, &m_pInstanceDataBuffer[i] ));

    // Make our one total buffer which is used for traditional instancing.
    bufferDesc.ByteWidth = m_numInstances * sizeof( InstanceData );
    V(m_pd3dDevice->CreateBuffer( &bufferDesc, NULL, &m_pTotalInstanceDataBuffer ));

	return hr;
}

HRESULT InstancingTests::UpdateInstanceData( float time )
{
	HRESULT hr = S_OK;

    if(!m_bTraditionalInstancing)
    {
        unsigned int buffersUsed = (m_numInstances / INTERNAL_MAX_INSTANCES)+1;
        for(unsigned int buffer = 0;buffer < buffersUsed ; buffer++)
        {
	        if(!m_pInstanceDataBuffer[buffer]) return E_INVALIDARG;

            D3D10_BUFFER_DESC bDesc;
            m_pInstanceDataBuffer[buffer]->GetDesc(&bDesc);
	        assert(bDesc.ByteWidth == INTERNAL_MAX_INSTANCES*sizeof(InstanceData));

            m_pd3dDevice->UpdateSubresource(m_pInstanceDataBuffer[buffer],D3D10CalcSubresource(0,0,1),NULL,(void*)((byte*)m_instanceData + buffer*bDesc.ByteWidth),0,0);
        }
    }
    else
    {
        D3D10_BUFFER_DESC bDesc;
        m_pTotalInstanceDataBuffer->GetDesc(&bDesc);

        // If wrong size, make a new one...
        if(bDesc.ByteWidth != sizeof(InstanceData)*m_numInstances)
        {
            SAFE_RELEASE(m_pTotalInstanceDataBuffer);
	        D3D10_BUFFER_DESC bufferDesc =
	        {
		        m_numInstances * sizeof( InstanceData ),
		        D3D10_USAGE_DEFAULT,
		        D3D10_BIND_VERTEX_BUFFER|D3D10_BIND_SHADER_RESOURCE,
		        0,
		        0
	        };
            V(m_pd3dDevice->CreateBuffer( &bufferDesc, NULL, &m_pTotalInstanceDataBuffer ));
        }
        m_pd3dDevice->UpdateSubresource(m_pTotalInstanceDataBuffer,D3D10CalcSubresource(0,0,1),NULL,(void*)m_instanceData,bDesc.ByteWidth,1);
    }

	return hr;
}	


void InstancingTests::Release()
{
    if(!m_pd3dDevice) return;

	SAFE_RELEASE(m_pd3dDevice);
	SAFE_RELEASE(m_pEffect10);
	SAFE_RELEASE(m_pInputLayout);
    SAFE_RELEASE(m_pInstancedInputLayout);
	SAFE_RELEASE(m_pMeshBuffer);
	SAFE_RELEASE(m_pMeshIndexBuffer);

    for(int i=0;i<INTERNAL_MAX_BUFFERS;i++)
	    SAFE_RELEASE(m_pInstanceDataBuffer[i]);

    SAFE_RELEASE( m_pTotalInstanceDataBuffer );

    SAFE_RELEASE(m_pDiffuseTexture);
    SAFE_RELEASE(m_pDiffuseSRV);
    SAFE_RELEASE(m_pNormalTexture);
    SAFE_RELEASE(m_pNormalSRV);
}

/*
	We encode matrices to have them take up less space in constants.  This decodes them for use.
*/
D3DXMATRIX InstancingTests::WorldMatrixFor(int instance)
{
	assert(instance <= INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES);
	D3DXMATRIX matrix; memset(&matrix,0,sizeof(D3DXMATRIX));
	D3DXVECTOR4 world1 = m_instanceData[instance].world1;
	D3DXVECTOR4 world2 = m_instanceData[instance].world2;
	D3DXVECTOR4 world3 = m_instanceData[instance].world3;
	matrix._11 = world1.x;matrix._12 = world1.y;matrix._13 = world1.z;matrix._41 = world1.w;
	matrix._21 = world2.x;matrix._22 = world2.y;matrix._23 = world2.z;matrix._42 = world2.w;
	matrix._31 = world3.x;matrix._32 = world3.y;matrix._33 = world3.z;matrix._43 = world3.w;
    matrix._44 = 1.f;
	return matrix;
}

/*
	This encodes the matrix into the constants
*/
void InstancingTests::SetWorldMatrixFor(int instance, D3DXMATRIX &mWorld)
{
	m_instanceData[instance].world1 = D3DXVECTOR4(mWorld._11,mWorld._12,mWorld._13,mWorld._41);
	m_instanceData[instance].world2 = D3DXVECTOR4(mWorld._21,mWorld._22,mWorld._23,mWorld._42);
	m_instanceData[instance].world3 = D3DXVECTOR4(mWorld._31,mWorld._32,mWorld._33,mWorld._43);
}

void InstancingTests::LoadTexture(const wchar_t * filename,ID3D10Texture2D**ppTexture10,ID3D10ShaderResourceView**ppSRV10)
{
    HRESULT hr;
    WCHAR str[MAX_PATH];
    if( SUCCEEDED(DXUTFindDXSDKMediaFileCch( str, MAX_PATH, filename )) )
    {
        ID3D10Resource *pRes = NULL;
        hr = D3DX10CreateTextureFromFile( m_pd3dDevice, str, NULL, NULL, &pRes, &hr );
        if( SUCCEEDED(hr) && pRes )
        {
            pRes->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)ppTexture10 );
            D3D10_TEXTURE2D_DESC desc;
            (*ppTexture10)->GetDesc( &desc );
            D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;
            ZeroMemory( &SRVDesc, sizeof(SRVDesc) );
            SRVDesc.Format = desc.Format;
            SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
            SRVDesc.Texture2D.MipLevels = desc.MipLevels;
            m_pd3dDevice->CreateShaderResourceView( *ppTexture10, &SRVDesc, ppSRV10 );
            SAFE_RELEASE( pRes );
        }
    }
}