//----------------------------------------------------------------------------------
// File:   InstancingTests.cpp
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#ifndef MAX_INSTANCE_CONSTANTS
#define MAX_INSTANCE_CONSTANTS 682
#endif


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

float4x4 g_mWorld : World;                  // World matrix for object
float4x4 g_mWorldViewProjection : WorldViewProjection;    // World * View * Projection matrix
float g_TexOffset : TEXCOORDANIMFACTOR;
float3 g_ViewPos : VIEWPOSITION;

struct PerInstanceData
{
	float4 world1;
	float4 world2;
	float4 world3;
};

cbuffer cInstanceData
{
	PerInstanceData	g_Instances[MAX_INSTANCE_CONSTANTS];
}

cbuffer cimmutable
{
	float3 g_lightPos = float3(0,50,150);
	float4x4 g_Identity = {	1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};
};


//--------------------------------------------------------------------------------------
// RenderStates
//--------------------------------------------------------------------------------------
RasterizerState Wire
{
	FillMode=Wireframe;
	CullMode=None;
};

//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
Texture2D g_txDiffuse		: DIFFUSE;
Texture2D g_txNormals		: NORMALMAP;

SamplerState g_samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};
SamplerState g_samAniso
{
    Filter = ANISOTROPIC;
    AddressU = Wrap;
    AddressV = Wrap;
};
SamplerState g_samPoint
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = Clamp;
	AddressV = Clamp;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_to_PS
{
	float4 pos		: SV_Position;
	float3 color	: COLOR;
	float2 tex		: TEXTURE0;
	float3 norm		: NORMAL;
	float3 tangent  : TANGENT;
	float3 binormal : BINORMAL;
	float3 worldPos : WORLDPOS;
	float3 view		: VIEW;
};

float4x4 decodeMatrix(float3x4 encodedMatrix)
{
	return float4x4(	float4(encodedMatrix[0].xyz,0),
						float4(encodedMatrix[1].xyz,0),
						float4(encodedMatrix[2].xyz,0),
						float4(encodedMatrix[0].w,encodedMatrix[1].w,encodedMatrix[2].w,1));
}

//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------

VS_to_PS InstancedConstantsVS( 
						// regular mesh data
						float3	vPos			: POSITION,
                        float3	vNormal			: NORMAL,
                        float2	vTexCoord0		: TEXCOORD0,
                        float3	vTangent		: TANGENT,
                        float3	vBiNormal		: BITANGENT,
						uint	InstanceId		: SV_InstanceID)
{
	VS_to_PS output;
	float3 vNormalWorldSpace;
	
	// Our per instance data is stored in constants
	float4 worldMatrix1 = g_Instances[InstanceId].world1;
	float4 worldMatrix2 = g_Instances[InstanceId].world2;
	float4 worldMatrix3 = g_Instances[InstanceId].world3;
		
	// the whole model position, decode a little, we have compressed for space.
	float4 vAnimatedPos = mul(float4(vPos.xyz,1), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));
	float4 vAnimatedNormal = mul(float4(vNormal.xyz,0), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));
	float4 vAnimatedTangent = mul(float4(vTangent.xyz,0), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));
	float4 vAnimatedBinormal = mul(float4(vBiNormal.xyz,0), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));

	// Transform the position from object space to homogeneous projection space
	output.pos = mul(vAnimatedPos, g_mWorldViewProjection);

	// Transform the normal from object space to world space    
	output.norm = vAnimatedNormal.xyz;
    output.tangent = vAnimatedTangent.xyz;
	output.binormal = vBiNormal.xyz;
	
	// Do the position too for lighting
	output.worldPos = float4(vAnimatedPos.xyz,1);
	output.tex.xy = float2(vTexCoord0.x,-vTexCoord0.y); 
	
	output.view = normalize(g_ViewPos - output.worldPos);
	
	output.color = float4(1,1,1,0);
	
	return output;
}

VS_to_PS InstancedVS( 
						// regular mesh data
						float3	vPos			: POSITION,
                        float3	vNormal			: NORMAL,
                        float2	vTexCoord0		: TEXCOORD0,
                        float3	vTangent		: TANGENT,
                        float3	vBiNormal		: BITANGENT,
                        float4  vWorld0          : WORLD0,
                        float4  vWorld1          : WORLD1,
                        float4  vWorld2          : WORLD2,
                        uint	InstanceId		: SV_InstanceID)
{
	VS_to_PS output;
	float3 vNormalWorldSpace;
	
	// Our per instance data is stored in constants
	float4 worldMatrix1 = vWorld0;
	float4 worldMatrix2 = vWorld1;
	float4 worldMatrix3 = vWorld2;
		
	// the whole model position, decode a little, we have compressed for space.
	float4 vAnimatedPos = mul(float4(vPos.xyz,1), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));
	float4 vAnimatedNormal = mul(float4(vNormal.xyz,0), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));
	float4 vAnimatedTangent = mul(float4(vTangent.xyz,0), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));
	float4 vAnimatedBinormal = mul(float4(vBiNormal.xyz,0), decodeMatrix(float3x4(worldMatrix1,worldMatrix2,worldMatrix3)));

	// Transform the position from object space to homogeneous projection space
	output.pos = mul(vAnimatedPos, g_mWorldViewProjection);

	// Transform the normal from object space to world space    
	output.norm = vAnimatedNormal.xyz;
    output.tangent = vAnimatedTangent.xyz;
	output.binormal = vBiNormal.xyz;
	
	// Do the position too for lighting
	output.worldPos = float4(vAnimatedPos.xyz,1);
	output.tex.xy = float2(vTexCoord0.x,-vTexCoord0.y); 
	
	output.view = normalize(g_ViewPos - output.worldPos);
	
	output.color = float4(1,1,1,0);
	
	return output;
}




VS_to_PS NormalVS(		
						float3	vPos			: POSITION,
                        float3	vNormal			: NORMAL,
                        float2	vTexCoord0		: TEXCOORD0,
                        float3	vTangent		: TANGENT,
                        float3	vBiNormal		: BITANGENT)
{
	VS_to_PS output;
	float3 vNormalWorldSpace;
	
	// Transform the position from object space to homogeneous projection space
	output.pos = mul(float4(vPos.xyz,1), g_mWorldViewProjection);

	// Transform the normal from object space to world space    
	output.norm = normalize(mul(vNormal, (float3x3)g_mWorld)); // normal (world space)
	output.tangent = normalize(mul(vTangent, (float3x3)g_mWorld)); // normal (world space)
	output.binormal = normalize(mul(vBiNormal, (float3x3)g_mWorld)); // normal (world space)
	
	// Do the position too for lighting
	output.worldPos = mul(float4(vPos.xyz,1), g_mWorld);
	output.tex.xy = float2(vTexCoord0.x,-vTexCoord0.y); 
	
	output.color = float4(1,1,1,0);
	
	output.view = normalize(g_ViewPos - output.worldPos);
	
	return output;    
}


//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
BlendState Opaque
{
	BlendEnable[0] = false;
};

float Kd = 0.89;
float Ks = 0.97;
float Kb = 2.0;
float SpecExpon = 24.0;
float4 ambiColor = float4(0.4,0.4,0.4,0);

float4 PS( VS_to_PS input) : SV_Target
{
    float4 albedo = g_txDiffuse.Sample(g_samAniso,input.tex.xy);	
    float4 bumps = g_txNormals.Sample(g_samAniso,input.tex.xy);    
    bumps = normalize(bumps);
    
    // lighting
    float3 Nn = normalize(input.norm);
    float3 Tn = normalize(input.tangent);
    float3 Bn = normalize(input.binormal);
    float3x3 mTangentToWorld = float3x3(Tn,Bn,Nn);
//    float3 Nb = mul( bumps.xyz, mTangentToWorld );
    float3 Nb = Nn + Kb*(bumps.x * Tn + bumps.y * Bn);
    Nb = normalize(Nb);
    
    float3 lightVec = normalize(g_lightPos - input.worldPos);    
    float3 Hn = normalize(normalize(input.view) + lightVec);
    float hdn = dot(Hn,Nb);
    float ldn = dot(lightVec,Nb);
    float4 litVec = lit(ldn,hdn,SpecExpon);
        
    float3 diffContrib = Kd * albedo.xyz * litVec.y;
    float3 specContrib = (litVec.z * Ks) * float4(0.1,0.1,0.7,0);
	
	return float4(ambiColor*albedo.xyz + diffContrib + specContrib,0);
	
}

//--------------------------------------------------------------------------------------
// Renders scene to Render target using D3D10 Techniques
//--------------------------------------------------------------------------------------
technique10 RenderSceneInstanced
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, InstancedVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        SetBlendState(Opaque,float4(0,0,0,0),0xffffffff);
        //SetRasterizerState(Wire);
    }
}

technique10 RenderSceneInstancedConstants
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, InstancedConstantsVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        SetBlendState(Opaque,float4(0,0,0,0),0xffffffff);
        //SetRasterizerState(Wire);
    }
}

technique10 RenderSceneNormal
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, NormalVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        SetBlendState(Opaque,float4(0,0,0,0),0xffffffff);        
		//SetRasterizerState(Wire);        
    }
}
